import { Request, Response } from "express";
import VehicleService from "../../../service/unit/vehicle/vehicleService";
import VehicleFactory from "../../../factory/admin/unit/vehicle/vehicle";
import {
  CreateVehicleCommand,
  DeleteVehicleCommand,
  UpdateVehicleCommand,
} from "../../../command/unit/vehicle/vehicleCommand";
import VehicleCommandHandler from "../../../command/unit/vehicle/vehicleCommandHandler";

/**
 * @description get all vehicles
 * @param req {Request} Express req object
 * @param res {Response} Express res object
 * @returns {Promise<*>}
 */
export async function getAllVehicles(req: Request, res: Response): Promise<any> {
  let offset = parseInt((req.query.offset as string) ?? "0");
  let count = parseInt((req.query.count as string) ?? "25");
  let search = (req.query.search as string) ?? "";
  let noLimit = req.query.noLimit === "true";
  let ids = ((req.query.ids ?? "") as string).split(",").filter((i) => i);

  //{ offset, count, search, noLimit, ids }
  let [vehicles, total] = await VehicleService.getAll();

  res.json({
    vehicles: vehicles,
    total: total,
    offset: offset,
    count: count,
  });
}

/**
 * @description get vehicle by id
 * @param req {Request} Express req object
 * @param res {Response} Express res object
 * @returns {Promise<*>}
 */
export async function getVehicleById(req: Request, res: Response): Promise<any> {
  const vehicleId = req.params.id;
  let vehicle = await VehicleService.getById(vehicleId);

  res.json(VehicleFactory.mapToSingle(vehicle));
}

/**
 * @description create vehicle
 * @param req {Request} Express req object
 * @param res {Response} Express res object
 * @returns {Promise<*>}
 */
export async function createVehicle(req: Request, res: Response): Promise<any> {
  const salutationId = parseInt(req.body.salutationId);

  let createVehicle: CreateVehicleCommand = {
    name: "",
    location: "",
    commissioned: undefined,
    vehicleTypeId: "",
  };
  let vehicleId = await VehicleCommandHandler.create(createVehicle);

  res.status(200).send(vehicleId);
}

/**
 * @description update vehicle by id
 * @param req {Request} Express req object
 * @param res {Response} Express res object
 * @returns {Promise<*>}
 */
export async function updateVehicleById(req: Request, res: Response): Promise<any> {
  const vehicleId = req.params.id;
  const salutationId = parseInt(req.body.salutationId);

  let updateVehicle: UpdateVehicleCommand = {
    id: vehicleId,
    name: "",
    location: "",
    commissioned: undefined,
  };
  await VehicleCommandHandler.update(updateVehicle);

  res.sendStatus(204);
}

/**
 * @description delete vehicle by id
 * @param req {Request} Express req object
 * @param res {Response} Express res object
 * @returns {Promise<*>}
 */
export async function deleteVehicleById(req: Request, res: Response): Promise<any> {
  const vehicleId = req.params.id;

  let deleteVehicle: DeleteVehicleCommand = {
    id: vehicleId,
  };
  await VehicleCommandHandler.delete(deleteVehicle);

  res.sendStatus(204);
}
